﻿using System;
using System.Collections.Generic;
using System.Text;

namespace WawaSoft.HttpStack
{
    public enum MessageType
    {
        UnKnow,
        Request,
        Response,
    }

    public class HttpMessage
    {
        public const string PROTOCOL = "HTTP";
        public const string CONTENT_LENGTH_HEADER = "Content-Length";
        public MessageType MessageType = MessageType.UnKnow;
        public BytesNode BodyStr;
        public int ContentLength;
        public Dictionary<string, string> Headers = new Dictionary<string, string>();
        internal BytesNode HeaderStr;
        public string Protocol;
        private string startLine;
        public object SyncRoot = new object(); //todo:暂时没用

        public HttpMessage()
        {
        }

        public HttpMessage(HttpMessage message)
        {
            startLine = message.startLine;
            Headers = message.Headers;
            BodyStr = message.BodyStr;
        }

        internal void InitHeaders(List<BytesLine> lines)
        {
            if (MessageType == MessageType.UnKnow)
            {
                #region 解析MessageType,ContentLength及填充消息头
                for (int i = 0; i < lines.Count; i++)
                {
                    BytesLine line = lines[i];
                    if (i == 0)
                    {
                        string tempStr = line.FullString;
                        MessageType = tempStr.StartsWith(PROTOCOL)
                                       ?
                                           MessageType.Response
                                       : MessageType.Request;
                        startLine = tempStr;
                        continue;
                    }
                    if (line.Pos1 == -1) throw new ApplicationException("header line error:"
                        + line.FullString);
                    //todo:暂时不考虑多个同名的头
                    Headers[line.String1] = line.String2;
                    if (Headers.ContainsKey(CONTENT_LENGTH_HEADER))
                        ContentLength = int.Parse(Headers[CONTENT_LENGTH_HEADER].Trim());
                }

                #endregion
            }
        }

        internal HttpRequest AsRequest()
        {
            if (MessageType != MessageType.Request)
                throw new ApplicationException("this message is not request");
            HttpRequest request = new HttpRequest(this);
            string[] tempArr = startLine.Split(' ');
            if (tempArr.Length != 3) throw new ApplicationException("start line error:" + startLine);
            request.Method = tempArr[0].Trim();
            request.Uri = tempArr[1];
            request.Protocol = tempArr[2];
            if (!request.Protocol.StartsWith(PROTOCOL))
                throw new ApplicationException("Protocol error:" + request.Protocol);
            return request;
        }

        internal HttpResponse AsResponse()
        {
            if (MessageType != MessageType.Response)
                throw new ApplicationException("this message is not response");
            HttpResponse response = new HttpResponse(this);
            string[] tempArr = startLine.Split(' ');
            if (tempArr.Length != 3) throw new ApplicationException("start line error:" + startLine);
            response.Protocol = tempArr[0];
            if (!response.Protocol.StartsWith(PROTOCOL))
                throw new ApplicationException("Protocol error:" + response.Protocol);
            response.StatusCode = int.Parse(tempArr[1].Trim()); //todo:可能有200.1这样的应答
            response.Desc = tempArr[2]; //todo:不考虑应答描述包含空格的情况

            return response;
        }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(startLine);
            sb.AppendLine();
            foreach (KeyValuePair<string,string> pair in Headers)
            {
                sb.AppendFormat("{0}:{1}", pair.Key, pair.Value);
                sb.AppendLine();
            }
            sb.AppendLine();
            sb.AppendLine();
            if (BodyStr != null) sb.Append(Encoding.UTF8.GetString(BodyStr.Bytes));
            return sb.ToString();
        }
    }
}